/**
 * A Javascript object to encode and/or decode html characters using HTML or Numeric entities that handles double or partial encoding
 * Author: R Reid
 * source: http://www.strictly-software.com/htmlencode
 * Licences: GPL, The MIT License (MIT)
 * Copyright: (c) 2011 Robert Reid - Strictly-Software.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 * Revision:
 *  2011-07-14, Jacques-Yves Bleau: 
 *       - fixed conversion error with capitalized accentuated characters
 *       + converted arr1 and arr2 to object property to remove redundancy
 *
 * Revision:
 *  2011-11-10, Ce-Yi Hio: 
 *       - fixed conversion error with a number of capitalized entity characters
 *
 * Revision:
 *  2011-11-10, Rob Reid: 
 *		 - changed array format
 */

Encoder = {

	// When encoding do we convert characters into html or numerical entities
	EncodeType : "entity",  // entity OR numerical

	isEmpty : function(val){
		if(val){
			return ((val===null) || val.length==0 || /^\s+$/.test(val));
		}else{
			return true;
		}
	},
	
	// arrays for conversion from HTML Entities to Numerical values
	arr1: ['&nbsp;','&iexcl;','&cent;','&pound;','&curren;','&yen;','&brvbar;','&sect;','&uml;','&copy;','&ordf;','&laquo;','&not;','&shy;','&reg;','&macr;','&deg;','&plusmn;','&sup2;','&sup3;','&acute;','&micro;','&para;','&middot;','&cedil;','&sup1;','&ordm;','&raquo;','&frac14;','&frac12;','&frac34;','&iquest;','&Agrave;','&Aacute;','&Acirc;','&Atilde;','&Auml;','&Aring;','&AElig;','&Ccedil;','&Egrave;','&Eacute;','&Ecirc;','&Euml;','&Igrave;','&Iacute;','&Icirc;','&Iuml;','&ETH;','&Ntilde;','&Ograve;','&Oacute;','&Ocirc;','&Otilde;','&Ouml;','&times;','&Oslash;','&Ugrave;','&Uacute;','&Ucirc;','&Uuml;','&Yacute;','&THORN;','&szlig;','&agrave;','&aacute;','&acirc;','&atilde;','&auml;','&aring;','&aelig;','&ccedil;','&egrave;','&eacute;','&ecirc;','&euml;','&igrave;','&iacute;','&icirc;','&iuml;','&eth;','&ntilde;','&ograve;','&oacute;','&ocirc;','&otilde;','&ouml;','&divide;','&oslash;','&ugrave;','&uacute;','&ucirc;','&uuml;','&yacute;','&thorn;','&yuml;','&quot;','&amp;','&lt;','&gt;','&OElig;','&oelig;','&Scaron;','&scaron;','&Yuml;','&circ;','&tilde;','&ensp;','&emsp;','&thinsp;','&zwnj;','&zwj;','&lrm;','&rlm;','&ndash;','&mdash;','&lsquo;','&rsquo;','&sbquo;','&ldquo;','&rdquo;','&bdquo;','&dagger;','&Dagger;','&permil;','&lsaquo;','&rsaquo;','&euro;','&fnof;','&Alpha;','&Beta;','&Gamma;','&Delta;','&Epsilon;','&Zeta;','&Eta;','&Theta;','&Iota;','&Kappa;','&Lambda;','&Mu;','&Nu;','&Xi;','&Omicron;','&Pi;','&Rho;','&Sigma;','&Tau;','&Upsilon;','&Phi;','&Chi;','&Psi;','&Omega;','&alpha;','&beta;','&gamma;','&delta;','&epsilon;','&zeta;','&eta;','&theta;','&iota;','&kappa;','&lambda;','&mu;','&nu;','&xi;','&omicron;','&pi;','&rho;','&sigmaf;','&sigma;','&tau;','&upsilon;','&phi;','&chi;','&psi;','&omega;','&thetasym;','&upsih;','&piv;','&bull;','&hellip;','&prime;','&Prime;','&oline;','&frasl;','&weierp;','&image;','&real;','&trade;','&alefsym;','&larr;','&uarr;','&rarr;','&darr;','&harr;','&crarr;','&lArr;','&uArr;','&rArr;','&dArr;','&hArr;','&forall;','&part;','&exist;','&empty;','&nabla;','&isin;','&notin;','&ni;','&prod;','&sum;','&minus;','&lowast;','&radic;','&prop;','&infin;','&ang;','&and;','&or;','&cap;','&cup;','&int;','&there4;','&sim;','&cong;','&asymp;','&ne;','&equiv;','&le;','&ge;','&sub;','&sup;','&nsub;','&sube;','&supe;','&oplus;','&otimes;','&perp;','&sdot;','&lceil;','&rceil;','&lfloor;','&rfloor;','&lang;','&rang;','&loz;','&spades;','&clubs;','&hearts;','&diams;'],
	arr2: ['&#160;','&#161;','&#162;','&#163;','&#164;','&#165;','&#166;','&#167;','&#168;','&#169;','&#170;','&#171;','&#172;','&#173;','&#174;','&#175;','&#176;','&#177;','&#178;','&#179;','&#180;','&#181;','&#182;','&#183;','&#184;','&#185;','&#186;','&#187;','&#188;','&#189;','&#190;','&#191;','&#192;','&#193;','&#194;','&#195;','&#196;','&#197;','&#198;','&#199;','&#200;','&#201;','&#202;','&#203;','&#204;','&#205;','&#206;','&#207;','&#208;','&#209;','&#210;','&#211;','&#212;','&#213;','&#214;','&#215;','&#216;','&#217;','&#218;','&#219;','&#220;','&#221;','&#222;','&#223;','&#224;','&#225;','&#226;','&#227;','&#228;','&#229;','&#230;','&#231;','&#232;','&#233;','&#234;','&#235;','&#236;','&#237;','&#238;','&#239;','&#240;','&#241;','&#242;','&#243;','&#244;','&#245;','&#246;','&#247;','&#248;','&#249;','&#250;','&#251;','&#252;','&#253;','&#254;','&#255;','&#34;','&#38;','&#60;','&#62;','&#338;','&#339;','&#352;','&#353;','&#376;','&#710;','&#732;','&#8194;','&#8195;','&#8201;','&#8204;','&#8205;','&#8206;','&#8207;','&#8211;','&#8212;','&#8216;','&#8217;','&#8218;','&#8220;','&#8221;','&#8222;','&#8224;','&#8225;','&#8240;','&#8249;','&#8250;','&#8364;','&#402;','&#913;','&#914;','&#915;','&#916;','&#917;','&#918;','&#919;','&#920;','&#921;','&#922;','&#923;','&#924;','&#925;','&#926;','&#927;','&#928;','&#929;','&#931;','&#932;','&#933;','&#934;','&#935;','&#936;','&#937;','&#945;','&#946;','&#947;','&#948;','&#949;','&#950;','&#951;','&#952;','&#953;','&#954;','&#955;','&#956;','&#957;','&#958;','&#959;','&#960;','&#961;','&#962;','&#963;','&#964;','&#965;','&#966;','&#967;','&#968;','&#969;','&#977;','&#978;','&#982;','&#8226;','&#8230;','&#8242;','&#8243;','&#8254;','&#8260;','&#8472;','&#8465;','&#8476;','&#8482;','&#8501;','&#8592;','&#8593;','&#8594;','&#8595;','&#8596;','&#8629;','&#8656;','&#8657;','&#8658;','&#8659;','&#8660;','&#8704;','&#8706;','&#8707;','&#8709;','&#8711;','&#8712;','&#8713;','&#8715;','&#8719;','&#8721;','&#8722;','&#8727;','&#8730;','&#8733;','&#8734;','&#8736;','&#8743;','&#8744;','&#8745;','&#8746;','&#8747;','&#8756;','&#8764;','&#8773;','&#8776;','&#8800;','&#8801;','&#8804;','&#8805;','&#8834;','&#8835;','&#8836;','&#8838;','&#8839;','&#8853;','&#8855;','&#8869;','&#8901;','&#8968;','&#8969;','&#8970;','&#8971;','&#9001;','&#9002;','&#9674;','&#9824;','&#9827;','&#9829;','&#9830;'],
		
	// Convert HTML entities into numerical entities
	HTML2Numerical : function(s){
		return this.swapArrayVals(s,this.arr1,this.arr2);
	},	

	// Convert Numerical entities into HTML entities
	NumericalToHTML : function(s){
		return this.swapArrayVals(s,this.arr2,this.arr1);
	},


	// Numerically encodes all unicode characters
	numEncode : function(s){
		
		if(this.isEmpty(s)) return "";

		var e = "";
		for (var i = 0; i < s.length; i++)
		{
			var c = s.charAt(i);
			if (c < " " || c > "~")
			{
				c = "&#" + c.charCodeAt() + ";";
			}
			e += c;
		}
		return e;
	},
	
	// HTML Decode numerical and HTML entities back to original values
	htmlDecode : function(s){

		var c,m,d = s;
		
		if(this.isEmpty(d)) return "";

		// convert HTML entites back to numerical entites first
		d = this.HTML2Numerical(d);
		
		// look for numerical entities &#34;
		arr=d.match(/&#[0-9]{1,5};/g);
		
		// if no matches found in string then skip
		if(arr!=null){
			for(var x=0;x<arr.length;x++){
				m = arr[x];
				c = m.substring(2,m.length-1); //get numeric part which is refernce to unicode character
				// if its a valid number we can decode
				if(c >= -32768 && c <= 65535){
					// decode every single match within string
					d = d.replace(m, String.fromCharCode(c));
				}else{
					d = d.replace(m, ""); //invalid so replace with nada
				}
			}			
		}

		return d;
	},		
	
	quitarHTML : function(cadena){
		var cadenaLimpia = cadena.replace(/(<([^>]+)>)/ig,"");
		return cadenaLimpia; 
	},

	// encode an input string into either numerical or HTML entities
	htmlEncode : function(s,dbl){
			
		if(this.isEmpty(s)) return "";

		// do we allow double encoding? E.g will &amp; be turned into &amp;amp;
		dbl = dbl || false; //default to prevent double encoding
		
		// if allowing double encoding we do ampersands first
		if(dbl){
			if(this.EncodeType=="numerical"){
				s = s.replace(/&/g, "&#38;");
			}else{
				s = s.replace(/&/g, "&amp;");
			}
		}

		// convert the xss chars to numerical entities ' " < >
		s = this.XSSEncode(s,false);
		
		if(this.EncodeType=="numerical" || !dbl){
			// Now call function that will convert any HTML entities to numerical codes
			s = this.HTML2Numerical(s);
		}

		// Now encode all chars above 127 e.g unicode
		s = this.numEncode(s);

		// now we know anything that needs to be encoded has been converted to numerical entities we
		// can encode any ampersands & that are not part of encoded entities
		// to handle the fact that I need to do a negative check and handle multiple ampersands &&&
		// I am going to use a placeholder

		// if we don't want double encoded entities we ignore the & in existing entities
		if(!dbl){
			s = s.replace(/&#/g,"##AMPHASH##");
		
			if(this.EncodeType=="numerical"){
				s = s.replace(/&/g, "&#38;");
			}else{
				s = s.replace(/&/g, "&amp;");
			}

			s = s.replace(/##AMPHASH##/g,"&#");
		}
		
		// replace any malformed entities
		s = s.replace(/&#\d*([^\d;]|$)/g, "$1");

		if(!dbl){
			// safety check to correct any double encoded &amp;
			s = this.correctEncoding(s);
		}

		// now do we need to convert our numerical encoded string into entities
		if(this.EncodeType=="entity"){
			s = this.NumericalToHTML(s);
		}

		return s;					
	},

	// Encodes the basic 4 characters used to malform HTML in XSS hacks
	XSSEncode : function(s,en){
		if(!this.isEmpty(s)){
			en = en || true;
			// do we convert to numerical or html entity?
			if(en){
				s = s.replace(/\'/g,"&#39;"); //no HTML equivalent as &apos is not cross browser supported
				s = s.replace(/\"/g,"&quot;");
				s = s.replace(/</g,"&lt;");
				s = s.replace(/>/g,"&gt;");
			}else{
				s = s.replace(/\'/g,"&#39;"); //no HTML equivalent as &apos is not cross browser supported
				s = s.replace(/\"/g,"&#34;");
				s = s.replace(/</g,"&#60;");
				s = s.replace(/>/g,"&#62;");
			}
			return s;
		}else{
			return "";
		}
	},

	// returns true if a string contains html or numerical encoded entities
	hasEncoded : function(s){
		if(/&#[0-9]{1,5};/g.test(s)){
			return true;
		}else if(/&[A-Z]{2,6};/gi.test(s)){
			return true;
		}else{
			return false;
		}
	},

	// will remove any unicode characters
	stripUnicode : function(s){
		return s.replace(/[^\x20-\x7E]/g,"");
		
	},

	// corrects any double encoded &amp; entities e.g &amp;amp;
	correctEncoding : function(s){
		return s.replace(/(&amp;)(amp;)+/,"$1");
	},


	// Function to loop through an array swaping each item with the value from another array e.g swap HTML entities with Numericals
	swapArrayVals : function(s,arr1,arr2){
		if(this.isEmpty(s)) return "";
		var re;
		if(arr1 && arr2){
			//ShowDebug("in swapArrayVals arr1.length = " + arr1.length + " arr2.length = " + arr2.length)
			// array lengths must match
			if(arr1.length == arr2.length){
				for(var x=0,i=arr1.length;x<i;x++){
					re = new RegExp(arr1[x], 'g');
					s = s.replace(re,arr2[x]); //swap arr1 item with matching item from arr2	
				}
			}
		}
		return s;
	},

	inArray : function( item, arr ) {
		for ( var i = 0, x = arr.length; i < x; i++ ){
			if ( arr[i] === item ){
				return i;
			}
		}
		return -1;
	},
	
	 quitarHTML : function(cadena){
	  var cadenaLimpia = cadena.replace(/(<([^>]+)>)/ig,"");
	  return cadenaLimpia; 
	 },
	 
	 dump: function (obj) {
	    var out = '';
	    for (var i in obj) {
	        out += i + ": " + obj[i] + "\n";
	    }
	
	    alert(out);
	},
	
	utf8_decode: function(str_data) {
	  // http://kevin.vanzonneveld.net
	  // +   original by: Webtoolkit.info (http://www.webtoolkit.info/)
	  // +      input by: Aman Gupta
	  // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
	  // +   improved by: Norman "zEh" Fuchs
	  // +   bugfixed by: hitwork
	  // +   bugfixed by: Onno Marsman
	  // +      input by: Brett Zamir (http://brett-zamir.me)
	  // +   bugfixed by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
	  // *     example 1: utf8_decode('Kevin van Zonneveld');
	  // *     returns 1: 'Kevin van Zonneveld'
	  var tmp_arr = [],
	    i = 0,
	    ac = 0,
	    c1 = 0,
	    c2 = 0,
	    c3 = 0;
	
	  str_data += '';
	
	  while (i < str_data.length) {
	    c1 = str_data.charCodeAt(i);
	    if (c1 < 128) {
	      tmp_arr[ac++] = String.fromCharCode(c1);
	      i++;
	    } else if (c1 > 191 && c1 < 224) {
	      c2 = str_data.charCodeAt(i + 1);
	      tmp_arr[ac++] = String.fromCharCode(((c1 & 31) << 6) | (c2 & 63));
	      i += 2;
	    } else {
	      c2 = str_data.charCodeAt(i + 1);
	      c3 = str_data.charCodeAt(i + 2);
	      tmp_arr[ac++] = String.fromCharCode(((c1 & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
	      i += 3;
	    }
	  }
	
	  return tmp_arr.join('');
	},
	
	utf8_encode: function (argString) {
	  // http://kevin.vanzonneveld.net
	  // +   original by: Webtoolkit.info (http://www.webtoolkit.info/)
	  // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
	  // +   improved by: sowberry
	  // +    tweaked by: Jack
	  // +   bugfixed by: Onno Marsman
	  // +   improved by: Yves Sucaet
	  // +   bugfixed by: Onno Marsman
	  // +   bugfixed by: Ulrich
	  // +   bugfixed by: Rafal Kukawski
	  // +   improved by: kirilloid
	  // *     example 1: utf8_encode('Kevin van Zonneveld');
	  // *     returns 1: 'Kevin van Zonneveld'
	
	  if (argString === null || typeof argString === "undefined") {
	    return "";
	  }
	
	  var string = (argString + ''); // .replace(/\r\n/g, "\n").replace(/\r/g, "\n");
	  var utftext = '',
	    start, end, stringl = 0;
	
	  start = end = 0;
	  stringl = string.length;
	  for (var n = 0; n < stringl; n++) {
	    var c1 = string.charCodeAt(n);
	    var enc = null;
	
	    if (c1 < 128) {
	      end++;
	    } else if (c1 > 127 && c1 < 2048) {
	      enc = String.fromCharCode((c1 >> 6) | 192, (c1 & 63) | 128);
	    } else {
	      enc = String.fromCharCode((c1 >> 12) | 224, ((c1 >> 6) & 63) | 128, (c1 & 63) | 128);
	    }
	    if (enc !== null) {
	      if (end > start) {
	        utftext += string.slice(start, end);
	      }
	      utftext += enc;
	      start = end = n + 1;
	    }
	  }
	
	  if (end > start) {
	    utftext += string.slice(start, stringl);
	  }
	
	  return utftext;
	},


	 
	 md5: function(str) {
		  // http://kevin.vanzonneveld.net
		  // +   original by: Webtoolkit.info (http://www.webtoolkit.info/)
		  // + namespaced by: Michael White (http://getsprink.com)
		  // +    tweaked by: Jack
		  // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
		  // +      input by: Brett Zamir (http://brett-zamir.me)
		  // +   bugfixed by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
		  // -    depends on: utf8_encode
		  // *     example 1: md5('Kevin van Zonneveld');
		  // *     returns 1: '6e658d4bfcb59cc13f96c14450ac40b9'
		  var xl;
		
		  var rotateLeft = function (lValue, iShiftBits) {
		    return (lValue << iShiftBits) | (lValue >>> (32 - iShiftBits));
		  };
		
		  var addUnsigned = function (lX, lY) {
		    var lX4, lY4, lX8, lY8, lResult;
		    lX8 = (lX & 0x80000000);
		    lY8 = (lY & 0x80000000);
		    lX4 = (lX & 0x40000000);
		    lY4 = (lY & 0x40000000);
		    lResult = (lX & 0x3FFFFFFF) + (lY & 0x3FFFFFFF);
		    if (lX4 & lY4) {
		      return (lResult ^ 0x80000000 ^ lX8 ^ lY8);
		    }
		    if (lX4 | lY4) {
		      if (lResult & 0x40000000) {
		        return (lResult ^ 0xC0000000 ^ lX8 ^ lY8);
		      } else {
		        return (lResult ^ 0x40000000 ^ lX8 ^ lY8);
		      }
		    } else {
		      return (lResult ^ lX8 ^ lY8);
		    }
		  };
		
		  var _F = function (x, y, z) {
		    return (x & y) | ((~x) & z);
		  };
		  var _G = function (x, y, z) {
		    return (x & z) | (y & (~z));
		  };
		  var _H = function (x, y, z) {
		    return (x ^ y ^ z);
		  };
		  var _I = function (x, y, z) {
		    return (y ^ (x | (~z)));
		  };
		
		  var _FF = function (a, b, c, d, x, s, ac) {
		    a = addUnsigned(a, addUnsigned(addUnsigned(_F(b, c, d), x), ac));
		    return addUnsigned(rotateLeft(a, s), b);
		  };
		
		  var _GG = function (a, b, c, d, x, s, ac) {
		    a = addUnsigned(a, addUnsigned(addUnsigned(_G(b, c, d), x), ac));
		    return addUnsigned(rotateLeft(a, s), b);
		  };
		
		  var _HH = function (a, b, c, d, x, s, ac) {
		    a = addUnsigned(a, addUnsigned(addUnsigned(_H(b, c, d), x), ac));
		    return addUnsigned(rotateLeft(a, s), b);
		  };
		
		  var _II = function (a, b, c, d, x, s, ac) {
		    a = addUnsigned(a, addUnsigned(addUnsigned(_I(b, c, d), x), ac));
		    return addUnsigned(rotateLeft(a, s), b);
		  };
		
		  var convertToWordArray = function (str) {
		    var lWordCount;
		    var lMessageLength = str.length;
		    var lNumberOfWords_temp1 = lMessageLength + 8;
		    var lNumberOfWords_temp2 = (lNumberOfWords_temp1 - (lNumberOfWords_temp1 % 64)) / 64;
		    var lNumberOfWords = (lNumberOfWords_temp2 + 1) * 16;
		    var lWordArray = new Array(lNumberOfWords - 1);
		    var lBytePosition = 0;
		    var lByteCount = 0;
		    while (lByteCount < lMessageLength) {
		      lWordCount = (lByteCount - (lByteCount % 4)) / 4;
		      lBytePosition = (lByteCount % 4) * 8;
		      lWordArray[lWordCount] = (lWordArray[lWordCount] | (str.charCodeAt(lByteCount) << lBytePosition));
		      lByteCount++;
		    }
		    lWordCount = (lByteCount - (lByteCount % 4)) / 4;
		    lBytePosition = (lByteCount % 4) * 8;
		    lWordArray[lWordCount] = lWordArray[lWordCount] | (0x80 << lBytePosition);
		    lWordArray[lNumberOfWords - 2] = lMessageLength << 3;
		    lWordArray[lNumberOfWords - 1] = lMessageLength >>> 29;
		    return lWordArray;
		  };
		
		  var wordToHex = function (lValue) {
		    var wordToHexValue = "",
		  wordToHexValue_temp = "",
		  lByte, lCount;
		for (lCount = 0; lCount <= 3; lCount++) {
		  lByte = (lValue >>> (lCount * 8)) & 255;
		  wordToHexValue_temp = "0" + lByte.toString(16);
		      wordToHexValue = wordToHexValue + wordToHexValue_temp.substr(wordToHexValue_temp.length - 2, 2);
		    }
		    return wordToHexValue;
		  };
		
		  var x = [],
		    k, AA, BB, CC, DD, a, b, c, d, S11 = 7,
		    S12 = 12,
		    S13 = 17,
		    S14 = 22,
		    S21 = 5,
		    S22 = 9,
		    S23 = 14,
		    S24 = 20,
		    S31 = 4,
		    S32 = 11,
		    S33 = 16,
		    S34 = 23,
		    S41 = 6,
		    S42 = 10,
		    S43 = 15,
		    S44 = 21;
		
		  str = this.utf8_encode(str);
		  x = convertToWordArray(str);
		  a = 0x67452301;
		  b = 0xEFCDAB89;
		  c = 0x98BADCFE;
		  d = 0x10325476;
		
		  xl = x.length;
		  for (k = 0; k < xl; k += 16) {
		    AA = a;
		    BB = b;
		    CC = c;
		    DD = d;
		    a = _FF(a, b, c, d, x[k + 0], S11, 0xD76AA478);
		    d = _FF(d, a, b, c, x[k + 1], S12, 0xE8C7B756);
		    c = _FF(c, d, a, b, x[k + 2], S13, 0x242070DB);
		    b = _FF(b, c, d, a, x[k + 3], S14, 0xC1BDCEEE);
		    a = _FF(a, b, c, d, x[k + 4], S11, 0xF57C0FAF);
		    d = _FF(d, a, b, c, x[k + 5], S12, 0x4787C62A);
		    c = _FF(c, d, a, b, x[k + 6], S13, 0xA8304613);
		    b = _FF(b, c, d, a, x[k + 7], S14, 0xFD469501);
		    a = _FF(a, b, c, d, x[k + 8], S11, 0x698098D8);
		    d = _FF(d, a, b, c, x[k + 9], S12, 0x8B44F7AF);
		    c = _FF(c, d, a, b, x[k + 10], S13, 0xFFFF5BB1);
		    b = _FF(b, c, d, a, x[k + 11], S14, 0x895CD7BE);
		    a = _FF(a, b, c, d, x[k + 12], S11, 0x6B901122);
		    d = _FF(d, a, b, c, x[k + 13], S12, 0xFD987193);
		    c = _FF(c, d, a, b, x[k + 14], S13, 0xA679438E);
		    b = _FF(b, c, d, a, x[k + 15], S14, 0x49B40821);
		    a = _GG(a, b, c, d, x[k + 1], S21, 0xF61E2562);
		    d = _GG(d, a, b, c, x[k + 6], S22, 0xC040B340);
		    c = _GG(c, d, a, b, x[k + 11], S23, 0x265E5A51);
		    b = _GG(b, c, d, a, x[k + 0], S24, 0xE9B6C7AA);
		    a = _GG(a, b, c, d, x[k + 5], S21, 0xD62F105D);
		    d = _GG(d, a, b, c, x[k + 10], S22, 0x2441453);
		    c = _GG(c, d, a, b, x[k + 15], S23, 0xD8A1E681);
		    b = _GG(b, c, d, a, x[k + 4], S24, 0xE7D3FBC8);
		    a = _GG(a, b, c, d, x[k + 9], S21, 0x21E1CDE6);
		    d = _GG(d, a, b, c, x[k + 14], S22, 0xC33707D6);
		    c = _GG(c, d, a, b, x[k + 3], S23, 0xF4D50D87);
		    b = _GG(b, c, d, a, x[k + 8], S24, 0x455A14ED);
		    a = _GG(a, b, c, d, x[k + 13], S21, 0xA9E3E905);
		    d = _GG(d, a, b, c, x[k + 2], S22, 0xFCEFA3F8);
		    c = _GG(c, d, a, b, x[k + 7], S23, 0x676F02D9);
		    b = _GG(b, c, d, a, x[k + 12], S24, 0x8D2A4C8A);
		    a = _HH(a, b, c, d, x[k + 5], S31, 0xFFFA3942);
		    d = _HH(d, a, b, c, x[k + 8], S32, 0x8771F681);
		    c = _HH(c, d, a, b, x[k + 11], S33, 0x6D9D6122);
		    b = _HH(b, c, d, a, x[k + 14], S34, 0xFDE5380C);
		    a = _HH(a, b, c, d, x[k + 1], S31, 0xA4BEEA44);
		    d = _HH(d, a, b, c, x[k + 4], S32, 0x4BDECFA9);
		    c = _HH(c, d, a, b, x[k + 7], S33, 0xF6BB4B60);
		    b = _HH(b, c, d, a, x[k + 10], S34, 0xBEBFBC70);
		    a = _HH(a, b, c, d, x[k + 13], S31, 0x289B7EC6);
		    d = _HH(d, a, b, c, x[k + 0], S32, 0xEAA127FA);
		    c = _HH(c, d, a, b, x[k + 3], S33, 0xD4EF3085);
		    b = _HH(b, c, d, a, x[k + 6], S34, 0x4881D05);
		    a = _HH(a, b, c, d, x[k + 9], S31, 0xD9D4D039);
		    d = _HH(d, a, b, c, x[k + 12], S32, 0xE6DB99E5);
		    c = _HH(c, d, a, b, x[k + 15], S33, 0x1FA27CF8);
		    b = _HH(b, c, d, a, x[k + 2], S34, 0xC4AC5665);
		    a = _II(a, b, c, d, x[k + 0], S41, 0xF4292244);
		    d = _II(d, a, b, c, x[k + 7], S42, 0x432AFF97);
		    c = _II(c, d, a, b, x[k + 14], S43, 0xAB9423A7);
		    b = _II(b, c, d, a, x[k + 5], S44, 0xFC93A039);
		    a = _II(a, b, c, d, x[k + 12], S41, 0x655B59C3);
		    d = _II(d, a, b, c, x[k + 3], S42, 0x8F0CCC92);
		    c = _II(c, d, a, b, x[k + 10], S43, 0xFFEFF47D);
		    b = _II(b, c, d, a, x[k + 1], S44, 0x85845DD1);
		    a = _II(a, b, c, d, x[k + 8], S41, 0x6FA87E4F);
		    d = _II(d, a, b, c, x[k + 15], S42, 0xFE2CE6E0);
		    c = _II(c, d, a, b, x[k + 6], S43, 0xA3014314);
		    b = _II(b, c, d, a, x[k + 13], S44, 0x4E0811A1);
		    a = _II(a, b, c, d, x[k + 4], S41, 0xF7537E82);
		    d = _II(d, a, b, c, x[k + 11], S42, 0xBD3AF235);
		    c = _II(c, d, a, b, x[k + 2], S43, 0x2AD7D2BB);
		    b = _II(b, c, d, a, x[k + 9], S44, 0xEB86D391);
		    a = addUnsigned(a, AA);
		    b = addUnsigned(b, BB);
		    c = addUnsigned(c, CC);
		    d = addUnsigned(d, DD);
		  }
		
		  var temp = wordToHex(a) + wordToHex(b) + wordToHex(c) + wordToHex(d);
		
		  return temp.toLowerCase();
	},
	
	utf8_decode: function (str_data) {
	  // http://kevin.vanzonneveld.net
	  // +   original by: Webtoolkit.info (http://www.webtoolkit.info/)
	  // +      input by: Aman Gupta
	  // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
	  // +   improved by: Norman "zEh" Fuchs
	  // +   bugfixed by: hitwork
	  // +   bugfixed by: Onno Marsman
	  // +      input by: Brett Zamir (http://brett-zamir.me)
	  // +   bugfixed by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
	  // *     example 1: utf8_decode('Kevin van Zonneveld');
	  // *     returns 1: 'Kevin van Zonneveld'
	  var tmp_arr = [],
	    i = 0,
	    ac = 0,
	    c1 = 0,
	    c2 = 0,
	    c3 = 0;
	
	  str_data += '';
	
	  while (i < str_data.length) {
	    c1 = str_data.charCodeAt(i);
	    if (c1 < 128) {
	      tmp_arr[ac++] = String.fromCharCode(c1);
	      i++;
	    } else if (c1 > 191 && c1 < 224) {
	      c2 = str_data.charCodeAt(i + 1);
	      tmp_arr[ac++] = String.fromCharCode(((c1 & 31) << 6) | (c2 & 63));
	      i += 2;
	    } else {
	      c2 = str_data.charCodeAt(i + 1);
	      c3 = str_data.charCodeAt(i + 2);
	      tmp_arr[ac++] = String.fromCharCode(((c1 & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
	      i += 3;
	    }
	  }
	
	  return tmp_arr.join('');
	},


};